#ifndef SUDOKU_H
#define SUDOKU_H

/* a sudoku is a 9x9 square of integers 1-9, or blanks represented by 0 */
typedef int sudoku_t[9][9];

/* the following definitions represent symmetries of a square. They
   are encoded as subgroups of the (8-element) group of symmetries */
#define SYM_NONE   0x001  /* no symmetry */
#define SYM_M_V    0x003  /* reflection, vertical axis */
#define SYM_M_H    0x021  /* reflection, horizontal axis */
#define SYM_M_D    0x009  /* reflection, diagonal */
#define SYM_M_C    0x081  /* reflection, codiagonal */
#define SYM_M_DC   0x099  /* reflection, diagonal+codiagonal */
#define SYM_M_VH   0x033  /* reflection, horizontal+vertical axis */
#define SYM_R_180  0x011  /* rotation, 180 degrees */
#define SYM_R_90   0x055  /* rotation, 90 degrees */
#define SYM_ALL    0x0ff  /* all symmetries */

#define MODE_COUNT    0x00
#define MODE_UNIQUE   0x01
#define MODE_FILL     0x02
#define MODE_CALLBACK 0x03
#define MODE_BRANCHES 0x04
#define MODE_MASK     0x0f
#define FLAG_RANDOM   0x10

/* callback function for sudoku_traverse. */
struct sudoku_callback_s {
  int (*fun)(sudoku_t su, void *privdata);
  void *privdata;
};
typedef struct sudoku_callback_s sudoku_callback_t;

/* General note: all functions in this API use rand(3) for randomness,
   so the behavior will be deterministic unless a seed is set first by
   the caller. */

/* search for solutions of the given sudoku, which may be partially
   filled-in already. The behavior depends on mode, which is one of
   the following values: MODE_COUNT (default): return the total number
   of solutions. MODE_UNIQUE: like MODE_COUNT, but stop counting after
   two solutions are found, i.e., return only 0, 1, or 2. MODE_FILL:
   fill su with the first solution found and return 1, or return 0 if
   no solution. MODE_BRANCHES: same as MODE_COUNT, but return total
   number of branches of the search tree, rather than number of
   solutions. (This is a measure of the difficulty of the puzzle when
   the solution is already known to be unique). MODE_CALLBACK:
   enumerate all solutions by calling the given callback function with
   each solution. Stop when the callback function's return value is 0.
   Moreover, if the mode is bitwise-or'ed with the value FLAG_RANDOM,
   then the solutions are enumerated in some random order. */
int sudoku_traverse(sudoku_t su, int mode, sudoku_callback_t *callback);

/* reduce the given sudoku, which means, create as many blanks as
   possible while keeping the solution unique and respecting the given
   symmetry. Does not accept SYM_RANDOM. */
void sudoku_reduce(sudoku_t su, int sym);

/* create a sudoku puzzle according to the given parameters. Return
   puzzle in su, and the answer in ans. Return value is the number of
   branches in the search tree (a measure of difficulty). If pattern
   is given, find a sudoku fitting the pattern. If reduce==1, reduce
   the answer. (Either a pattern or reduce or both must be
   specified). Sym is a symmetry type or SYM_RANDOM. */
int sudoku(sudoku_t su, sudoku_t ans, int pattern[9][9], int reduce, int sym);

/* print an ASCII representation of the sudoku to fout. This is a
   primitive output routine used by debugging code. */
void sudoku_print(sudoku_t su, FILE *fout);
void sudoku_print_big(sudoku_t su, FILE *fout);
void sudoku_print_small(sudoku_t su, FILE *fout);

#endif /* SUDOKU_H */
